@@ -22,12 +22,15 @@ function(sgp_object,
 		institution_level="GRADE",
 		demographic=c("GENDER", "ETHNICITY", "FREE_REDUCED_LUNCH_STATUS", "ELL_STATUS", "IEP_STATUS", "GIFTED_AND_TALENTED_PROGRAM_STATUS", "CATCH_UP_KEEP_UP_STATUS_INITIAL"),
 		institution_inclusion=list(STATE="STATE_ENROLLMENT_STATUS", DISTRICT_NUMBER="DISTRICT_ENROLLMENT_STATUS", SCHOOL_NUMBER="SCHOOL_ENROLLMENT_STATUS")),
-	confidence.interval.groups=list(institution="SCHOOL_NUMBER",
-		content="CONTENT_AREA",
-		time="YEAR",
-		institution_level=NULL,
-		demographic=NULL,
-		institution_inclusion=list(STATE=NULL, DISTRICT_NUMBER=NULL, SCHOOL_NUMBER="SCHOOL_ENROLLMENT_STATUS"))) {
+        confidence.interval.groups=list(TYPE="Bootstrap",
+                VARIABLES=c("SGP"),
+                QUANTILES=c(0.025, 0.975),
+                GROUPS=list(institution="SCHOOL_NUMBER",
+                content="CONTENT_AREA",
+                time="YEAR",
+                institution_level= NULL,
+                demographic=NULL,
+                institution_inclusion=list(STATE=NULL, DISTRICT_NUMBER=NULL, SCHOOL_NUMBER="SCHOOL_ENROLLMENT_STATUS")))) {
 
         started.at <- proc.time()
 	message(paste("Started abcSGP", date()), "\n")
